Player = Core.class(Sprite)

function Player:init(xworld, xspritesheet, xcols, xrows)
	-- settings
	self.world = xworld
	self.lives = 1
	self.speed = 4
	self.flip = 1
	self.numfeetcontacts = 0 -- is on ground
	self.numplatformcontacts = 0 -- is on platform
	self.isleft, self.isright, self.isdown = false, false, false -- controls
	self.isgoingdownplatform = false

	-- animations
	self.currentanim = ""
	self.frame = 0
	self.animspeed = 1 / 6
	self.animtimer = self.animspeed
	-- retrieve all anims in spritesheet
	local myanimstex = Texture.new(xspritesheet)
	local w, h = myanimstex:getWidth() / xcols, myanimstex:getHeight() / xrows
	local myanims_list = {}
	for r = 1, xrows do
		for c = 1, xcols do
			local myanimstexregion = TextureRegion.new(myanimstex, (c - 1) * w, (r - 1) * h, w, h)
			myanims_list[#myanims_list + 1] = myanimstexregion
		end
	end
	-- create anims 1
	self.anims = {}
	function createAnim(xanimname, xstart, xfinish, xanimslist)
		self.anims[xanimname] = {}
		for i = xstart, xfinish do
			self.anims[xanimname][#self.anims[xanimname] + 1] = xanimslist[i]
		end
	end
	-- create anims 2
	createAnim("idle", 1, 2, myanims_list)
	createAnim("jump_up", 38, 38, myanims_list)
	createAnim("jump_down", 40, 40, myanims_list)
	createAnim("walk", 13, 16, myanims_list)

	-- the sprite & the bitmap
	local mysprite = Sprite.new()
	self.bitmap = Bitmap.new(myanims_list[1]) -- starting bmp texture
	self.bitmap:setAnchorPoint(0.5, 0.5)
	self.bitmap:setPosition(w / 2, h / 2)
	mysprite:addChild(self.bitmap)
	self:addChild(mysprite)

	-- the dynamic body (liquidfun)
	self.body = xworld:createBody { type = b2.DYNAMIC_BODY }
--	self.body:setBullet(true)
	self.body:setFixedRotation(true)
	local x = 8
--	local bodyshape = b2.PolygonShape.new()
--	bodyshape:setAsBox(w / 4 - 12, h / 2, w / 2, h / 2, 0)
	local bodyshape = b2.CircleShape.new(w / 2, h - 15, 15)
	local bodyfixture = self.body:createFixture {
		shape = bodyshape, density = 1, restitution = 0, friction = 1
	}
	-- the left and right body with no friction so does not stick on walls!
	local myleftshape = b2.PolygonShape.new()
	myleftshape:setAsBox(2, h / 2 - 6, w / 4 + 2, h / 2 - 6 / 2, 0)
	local myleftfixture = self.body:createFixture { shape = myleftshape, friction = 0 }
	local myrightshape = b2.PolygonShape.new()
	myrightshape:setAsBox(2, h / 2 - 6, w - w / 4 - 2, h / 2 - 6 / 2, 0)
	local myrightfixture = self.body:createFixture { shape = myrightshape, friction = 0 }
	-- the feet sensor / is on ground
	local myfeetshape = b2.PolygonShape.new()
	myfeetshape:setAsBox(w / 4, 8, w / 2, h, 0)
	local myfeetfixture = self.body:createFixture { shape = myfeetshape, isSensor = true }
	myfeetfixture.type = 2

	-- keys listeners
	self:addEventListener(Event.ENTER_FRAME, self.onEnterFrame, self)
	self:addEventListener(Event.KEY_DOWN, self.onKeyDown, self)
	self:addEventListener(Event.KEY_UP, self.onKeyUp, self)
end

-- update player
function Player:onEnterFrame(e)
	local posx, posy = self.body:getPosition()
	local vx, vy = self.body:getLinearVelocity()

	-- anim state
	if self.numfeetcontacts > 0 then -- is on floor anims
		if vx == 0 then
			self.currentanim = "idle"
		elseif vx ~= 0 then
			if self.numplatformcontacts > 0 and not self.isleft and not self.isright then
				self.currentanim = "idle"
			else
				self.currentanim = "walk"
			end
		end
	else -- is not on floor anims
		if vy < 0 then
			self.currentanim = "jump_up"
		elseif vy >= 0 then
			self.currentanim = "jump_down"
		end
	end

	-- anim loop
	if self.currentanim ~= "" then
		self.animtimer = self.animtimer - e.deltaTime
		if self.animtimer <= 0 then
			self.frame += 1
			self.animtimer = self.animspeed
			if self.frame > #self.anims[self.currentanim] then
				self.frame = 1
			end
			self.bitmap:setTextureRegion(self.anims[self.currentanim][self.frame])
		end
	end

	-- player controls (gradual velocity)
	if self.isleft and not self.isright then
		vx = math.max(vx - 1, -self.speed)
		self.flip = -1
	elseif self.isright and not self.isleft then
		vx = math.min(vx + 1, self.speed)
		self.flip = 1
	end
	self.body:setLinearVelocity(vx, vy)

	-- move player sprite along with player physic body
	self:setPosition(posx, posy)
	self:setRotation(self.body:getAngle() * 180 / math.pi)
	self.bitmap:setScaleX(self.flip) -- left/right facing
end

-- functions
function Player:jump()
	if self.numfeetcontacts > 0 then -- is on ground
		local gx, gy = self.world:getGravity()
		self.body:setLinearVelocity(0, -gy / 1.9)
	end
end

function Player:shoot()
	local px, py = self.body:getPosition()
	local pmx = px + self:getWidth() / 2 -- get middle of sprite position
	self.bullet = LF_Bullet.new(self.world, "gfx/world/mytile32.png", 1, 0.5, true,
			0.1, 1, 0,
			24 * self.flip, 0, pmx + 38 * self.flip, py + self:getHeight() / 2,
			100)
	self:getParent():addChild(self.bullet)
end

-- keys listeners
function Player:onKeyDown(e)
	if e.keyCode == KeyCode.LEFT then self.isleft = true end
	if e.keyCode == KeyCode.RIGHT then self.isright = true end
	if e.keyCode == KeyCode.UP then self:jump() end -- ***
	if e.keyCode == KeyCode.DOWN then
		self.isdown = true
		self.isgoingdownplatform = true
		self.body:setAwake(true)
	end
	if e.keyCode == KeyCode.SPACE then self:shoot() end -- ***
end

function Player:onKeyUp(e)
	if e.keyCode == KeyCode.LEFT then self.isleft = false end
	if e.keyCode == KeyCode.RIGHT then self.isright = false end
	if e.keyCode == KeyCode.DOWN then self.isdown = false end
end
